import { Component, Host, h, Element, Prop, Watch } from '@stencil/core';
export class STooltip {
  constructor() {
    this._isTooltipEnabled = false;
    this.margin = 10;
    this.orientation = 'bottom';
    this.followMouse = false;
    this.noDefaultStyle = false;
    this.noArrow = false;
    this.color = 'white';
    this.backgroundColor = 'black';
    this.maxWidth = '500px';
    this.maxHeight = '300px';
    this.borderWidth = '0px';
    this.borderColor = 'white';
    this.opacity = .9;
    this.zIndex = 99999;
    this.shadow = '0px 5px 10px 0px grey';
  }
  get isTooltipEnabled() {
    return this._isTooltipEnabled;
  }
  set isTooltipEnabled(value) {
    if (value !== this._isTooltipEnabled) {
      this._isTooltipEnabled = value;
      this.updateCSSVariable('--tooltip-display', value ? 'block' : 'none');
      if (!value) {
        this.hostElement.shadowRoot.querySelector('#tooltip-container').innerHTML = '<slot/>';
      }
    }
  }
  get attachedElements() {
    if (this.attachTo) {
      if (typeof this.attachTo === 'string') {
        return document.querySelectorAll(this.attachTo);
      }
      else {
        return this.attachTo;
      }
    }
    else {
      return [this.hostElement.parentElement];
    }
  }
  updateColor(value) {
    this.updateCSSVariable('--tooltip-color', value);
  }
  updateBackgroundColor(value) {
    this.updateCSSVariable('--tooltip-background-color', value);
  }
  updateMaxWidth(value) {
    this.updateCSSVariable('--tooltip-max-width', value);
  }
  updateMaxHeight(value) {
    this.updateCSSVariable('--tooltip-max-height', value);
  }
  updateBorderWidth(value) {
    this.updateCSSVariable('--tooltip-border-width', value);
  }
  updateBorderColor(value) {
    this.updateCSSVariable('--tooltip-border-color', value);
  }
  updateOpacity(value) {
    this.updateCSSVariable('--tooltip-opacity', value.toString());
  }
  updateZIndex(value) {
    this.updateCSSVariable('--tooltip-z-index', value.toString());
  }
  updateShadow(value) {
    this.updateCSSVariable('--tooltip-shadow', value);
  }
  connectedCallback() {
    this.attachedElements.forEach(attachedElement => {
      attachedElement.addEventListener('mouseover', () => this.isTooltipEnabled = true);
      attachedElement.addEventListener('mouseout', () => this.isTooltipEnabled = false);
      attachedElement.addEventListener('mousemove', event => this.updateTooltipPosition(event));
    });
    this.initializeCSSVariables();
  }
  render() {
    return (h(Host, null,
      h("div", { id: "tooltip-container", class: [
          this.orientation,
          this.noDefaultStyle ? '' : 'styled',
          this.noArrow ? '' : 'arrow'
        ].join(' ') },
        h("slot", null))));
  }
  updateTooltipPosition(mouseEvent) {
    let x = 0, y = 0;
    if (this.followMouse) {
      x = mouseEvent.x;
      y = mouseEvent.y;
    }
    else {
      const targetElement = mouseEvent.target;
      const tooltipText = targetElement.getAttribute('data-s-tooltip-text');
      if (tooltipText) {
        this.hostElement.shadowRoot.querySelector('#tooltip-container').innerHTML = tooltipText;
      }
      const { top, right, bottom, left, width, height } = targetElement.getBoundingClientRect();
      switch (this.orientation) {
        case 'top':
          x = right - width / 2;
          y = top;
          break;
        case 'right':
          x = right;
          y = bottom - height / 2;
          break;
        case 'bottom':
          x = right - width / 2;
          y = bottom;
          break;
        case 'left':
          x = left;
          y = bottom - height / 2;
          break;
      }
    }
    this.setTooltipPosition(x, y);
  }
  setTooltipPosition(x, y) {
    const tooltipContainerElement = this.hostElement.shadowRoot.querySelector('#tooltip-container');
    const tooltipWidth = tooltipContainerElement.clientWidth;
    const tooltipHeight = tooltipContainerElement.clientHeight;
    let tooltipLeft = x;
    let tooltipTop = y;
    switch (this.orientation) {
      case 'top':
        tooltipLeft = x - tooltipWidth / 2;
        tooltipTop = y - tooltipHeight - this.margin;
        break;
      case 'right':
        tooltipLeft = x + this.margin;
        tooltipTop = y - tooltipHeight / 2;
        break;
      case 'bottom':
        tooltipLeft = x - tooltipWidth / 2;
        tooltipTop = y + this.margin;
        break;
      case 'left':
        tooltipLeft = x - tooltipWidth - this.margin;
        255;
        tooltipTop = y - tooltipHeight / 2;
        break;
    }
    this.updateCSSVariable('--tooltip-left', `${tooltipLeft}px`);
    this.updateCSSVariable('--tooltip-top', `${tooltipTop}px`);
  }
  initializeCSSVariables() {
    this.updateColor(this.color);
    this.updateBackgroundColor(this.backgroundColor);
    this.updateMaxWidth(this.maxWidth);
    this.updateMaxHeight(this.maxHeight);
    this.updateBorderWidth(this.borderWidth);
    this.updateBorderColor(this.borderColor);
    this.updateOpacity(this.opacity);
    this.updateZIndex(this.zIndex);
    this.updateShadow(this.shadow);
  }
  updateCSSVariable(variableName, value) {
    this.hostElement.style.setProperty(variableName, value);
  }
  static get is() { return "s-tooltip"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["s-tooltip.css"]
  }; }
  static get styleUrls() { return {
    "$": ["s-tooltip.css"]
  }; }
  static get properties() { return {
    "margin": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "margin",
      "reflect": true,
      "defaultValue": "10"
    },
    "orientation": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'top' | 'right' | 'bottom' | 'left'",
        "resolved": "\"bottom\" | \"left\" | \"right\" | \"top\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "orientation",
      "reflect": true,
      "defaultValue": "'bottom'"
    },
    "followMouse": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "follow-mouse",
      "reflect": true,
      "defaultValue": "false"
    },
    "noDefaultStyle": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "no-default-style",
      "reflect": true,
      "defaultValue": "false"
    },
    "noArrow": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "no-arrow",
      "reflect": true,
      "defaultValue": "false"
    },
    "attachTo": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string | HTMLElement[]",
        "resolved": "HTMLElement[] | string",
        "references": {
          "HTMLElement": {
            "location": "global"
          }
        }
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "attach-to",
      "reflect": true
    },
    "color": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "color",
      "reflect": true,
      "defaultValue": "'white'"
    },
    "backgroundColor": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "background-color",
      "reflect": true,
      "defaultValue": "'black'"
    },
    "maxWidth": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "max-width",
      "reflect": true,
      "defaultValue": "'500px'"
    },
    "maxHeight": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "max-height",
      "reflect": true,
      "defaultValue": "'300px'"
    },
    "borderWidth": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "border-width",
      "reflect": true,
      "defaultValue": "'0px'"
    },
    "borderColor": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "border-color",
      "reflect": true,
      "defaultValue": "'white'"
    },
    "opacity": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "opacity",
      "reflect": true,
      "defaultValue": ".9"
    },
    "zIndex": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "z-index",
      "reflect": true,
      "defaultValue": "99999"
    },
    "shadow": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "string",
        "resolved": "string",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "shadow",
      "reflect": true,
      "defaultValue": "'0px 5px 10px 0px grey'"
    }
  }; }
  static get elementRef() { return "hostElement"; }
  static get watchers() { return [{
      "propName": "color",
      "methodName": "updateColor"
    }, {
      "propName": "backgroundColor",
      "methodName": "updateBackgroundColor"
    }, {
      "propName": "maxWidth",
      "methodName": "updateMaxWidth"
    }, {
      "propName": "maxHeight",
      "methodName": "updateMaxHeight"
    }, {
      "propName": "borderWidth",
      "methodName": "updateBorderWidth"
    }, {
      "propName": "borderColor",
      "methodName": "updateBorderColor"
    }, {
      "propName": "opacity",
      "methodName": "updateOpacity"
    }, {
      "propName": "zIndex",
      "methodName": "updateZIndex"
    }, {
      "propName": "shadow",
      "methodName": "updateShadow"
    }]; }
}
